@@ -12,7 +12,8 @@ use crate::database::entry::{Entry, EntryValue, InvariantEntry};
 use crate::database::hierarchies::{resolve_path_cached, ResolveCache, UNode, UPath};
 use crate::database::inner::models;
 use crate::database::{
-    file_set_valid, insert_entry, insert_file, retrieve_all_files, DbPool, UPEND_SUBDIR,
+    file_set_valid, file_update_mtime, insert_entry, insert_file, retrieve_all_files, DbPool,
+    UPEND_SUBDIR,
 };
 use crate::util::hash::{Hash, Hashable};
 use crate::util::jobs::{Job, JobContainer, JobId, State};
@@ -77,6 +78,26 @@ pub async fn rescan_vault(
             .unwrap();
     }
 }
+struct PragmaSynchronousGuard<'a>(&'a DbPool);
+
+impl Drop for PragmaSynchronousGuard<'_> {
+    fn drop(&mut self) {
+        debug!("Re-enabling synchronous mode.");
+        let connection = self.0.get();
+        let res: Result<_, String> = match connection {
+            Ok(connection) => connection
+                .execute("PRAGMA synchronous = NORMAL;")
+                .map_err(|err| format!("{}", err)),
+            Err(err) => Err(format!("{}", err)),
+        };
+        if let Err(err) = res {
+            error!(
+                "Error setting synchronous mode back to NORMAL! Data loss possible! {}",
+                err
+            );
+        }
+    }
+}
 
 type UpdatePathResult = Result<UpdatePathOutcome>;
 
@@ -104,6 +125,7 @@ fn _rescan_vault<T: AsRef<Path>>(
     // Disable syncing in SQLite for the duration of the import
     debug!("Disabling SQLite synchronous mode");
     pool.get()?.execute("PRAGMA synchronous = OFF;")?;
+    let _guard = PragmaSynchronousGuard(&pool);
 
     // Walk through the vault, find all paths
     debug!("Traversing vault directory");
@@ -152,6 +174,8 @@ fn _rescan_vault<T: AsRef<Path>>(
         })
         .collect();
 
+    debug!("Processing done, cleaning up...");
+
     let existing_files = existing_files.read().unwrap();
 
     let connection = pool.get()?;
@@ -198,8 +222,7 @@ fn _rescan_vault<T: AsRef<Path>>(
     }
 
     // Re-enable SQLite syncing
-    debug!("Re-enabling synchronous mode.");
-    pool.get()?.execute("PRAGMA synchronous = NORMAL;")?;
+    drop(_guard);
 
     info!(
         "Finished updating {} ({} created, {} deleted, {} left unchanged). Took {}s.",
@@ -254,18 +277,24 @@ fn _process_directory_entry<P: AsRef<Path>>(
         let existing_file = existing_file.clone();
         drop(existing_files_read);
 
-        let mut same = size == existing_file.size && mtime == existing_file.mtime;
-        if !same {
-            file_hash = Some(path.hash()?);
-            same = file_hash.as_ref().unwrap() == &existing_file.hash;
-        }
+        if size == existing_file.size {
+            let same_mtime = mtime.is_some() && mtime == existing_file.mtime;
+            let mut same_hash = false;
 
-        if same {
-            if !existing_file.valid {
-                file_set_valid(connection, existing_file.id, true)?;
+            if !same_mtime {
+                file_hash = Some(path.hash()?);
+                same_hash = file_hash.as_ref().unwrap() == &existing_file.hash;
             }
 
-            {
+            if same_mtime || same_hash {
+                if mtime != existing_file.mtime {
+                    file_update_mtime(connection, existing_file.id, mtime)?;
+                }
+
+                if !existing_file.valid {
+                    file_set_valid(connection, existing_file.id, true)?;
+                }
+
                 let mut existing_files_write = existing_files.write().unwrap();
                 let maybe_existing_file = existing_files_write
                     .iter()
